from django.http.response import HttpResponse
import json


class OEmbedResponse(HttpResponse):
    def __init__(
        self, kind, width, height, html, version=1.0, title=None,
        jsonp_callback=None
    ):
        super().__init__()

        data = json.dumps(
            {
                'type': kind,
                'html': html,
                'width': width,
                'height': height,
                'version': version,
                'title': title
            }
        )

        if jsonp_callback:
            self.content = '%s(%s)' % (
                jsonp_callback,
                data
            )

            self['Content-Type'] = 'text/javascript'
        else:
            self.content = data
            self['Content-Type'] = 'application/json'



from hashlib import md5
from .caching import cache
from . import settings, utils


try:
    from django.utils.html import escape
except ImportError:
    try:
        from markupsafe import escape
    except ImportError:
        raise Exception(
            'Cannot find escape function in django.utils.html or markupsafe'
        )


class NotFoundError(Exception):
    pass


class CachedObject(object):
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if not hasattr(self, key):
                raise TypeError(
                    '__init__() got an unexpected keyword argument \'%s\'' % (
                        key
                    )
                )

            setattr(self, key, value)

    @property
    def _cachekey(self):
        return '%s%s' % (
            settings.CACHE_PREFIX,
            md5(str(self.id).encode('utf-8')).hexdigest()
        )

    @classmethod
    def load(cls, id):
        cachekey = '%s%s' % (
            settings.CACHE_PREFIX,
            md5(str(id).encode('utf-8')).hexdigest()
        )

        if cachekey in cache:
            obj = cache.get(cachekey)
            del obj['id']
            return cls(**obj)

        raise NotFoundError(
            'Object not found with ID %s' % id
        )

    @classmethod
    def create(cls, **kwargs):
        obj = cls(**kwargs)
        obj.save()
        return obj

    def save(self):
        cache.set(
            self._cachekey,
            self.to_dict(),
            settings.CACHE_TIMEOUT
        )

    def to_dict(self):
        return {
            'id': self.id
        }


class Resource(CachedObject):
    url = ''
    width = 0
    html = ''
    title = ''
    thumbnail = ''

    def to_dict(self):
        d = super().to_dict()
        d.update(
            {
                'url': self.url,
                'width': self.width,
                'html': self.html,
                'title': self.title,
                'thumbnail': self.thumbnail
            }
        )

        return d

    def __str__(self):  # pragma: no cover
        return self.url

    @property
    def id(self):
        return str(self.url)

    @classmethod
    def load(cls, url):
        try:
            return super().load(url)
        except NotFoundError:
            return cls.create(url)

    @classmethod
    def create(cls, url, width=settings.WIDTH):
        endpoint, fmt = utils.get_oembed_endpoint(url)

        if endpoint and fmt:
            html, title, thumbnail = utils.get_oembed_content(
                url,
                endpoint,
                fmt,
                width
            )
        else:
            html, title, thumbnail = utils.guess_oembed_content(url, width)
            html = utils.sandbox_iframe(html)

        return super().create(
            url=url,
            width=width,
            html=html,
            title=title,
            thumbnail=thumbnail
        )

    def to_html(self, link=False, thumbnail=False):
        if thumbnail and self.thumbnail:
            body = '<img src="%s">' % escape(self.thumbnail)
        else:
            body = self.title and escape(self.title) or escape(self.url)

        prefix, suffix = '', ''

        if link:
            prefix = '<a href="%s" target="_blank" title="%s">' % (
                escape(self.url),
                self.title and escape(self.title) or '',
            )

            suffix = '</a>'

        if link or thumbnail:
            return prefix + body + suffix

        return self.html



import json
import logging
import os
import re


try:
    from django.conf import settings
except ImportError:
    class SettingsObject(object):
        def __init__(self):
            for key in (
                'OEMBED_WIDTH',
                'OEMBED_SANDBOX',
                'OEMBED_URLPATTERNS',
                'OEMBED_CACHE_PREFIX',
                'OEMBED_CACHE_TIMEOUT',
                'OEMBED_AJAX_OBJECT_URL'
            ):
                if os.getenv(key):
                    setattr(self, key, os.getenv(key))

    settings = SettingsObject()


def load_providers():
    filename = os.path.join(
        os.path.dirname(__file__),
        'fixtures',
        'providers.json'
    )

    return json.load(open(filename, 'rb'))


WIDTH = getattr(settings, 'OEMBED_WIDTH', 640)
SANDBOX = getattr(settings, 'OEMBED_SANDBOX', False)
URL_PATTERNS = list(
    getattr(settings, 'OEMBED_URLPATTERNS', [])
) + load_providers()

CACHE_PREFIX = getattr(settings, 'OEMBED_CACHE_PREFIX', 'oembed_')
CACHE_TIMEOUT = getattr(
    settings,
    'OEMBED_CACHE_TIMEOUT',
    getattr(settings, 'CACHES', {}).get('default', {}).get('TIMEOUT', 300)
)

AJAX_OBJECT_URL = getattr(
    settings,
    'OEMBED_AJAX_OBJECT_URL',
    None
)

P_REGEX = re.compile(
    r'<p>([^<]+)<\/p>', re.IGNORECASE
)

LINK_REGEX = re.compile(r'\<link[^\>]+\>', re.IGNORECASE)
ATTR_REGEX = re.compile(
    r""" ([a-z]+)=(?:"([^"]+)"|'([^']+)')""",
    re.IGNORECASE
)

LINK_TYPE_REGEX = re.compile(r'^application/(json|xml)\+oembed$')
OEMBED_LOGGER = logging.getLogger('oembed')


__all__ = [
    'ATTR_REGEX',
    'CACHE_PREFIX',
    'CACHE_TIMEOUT',
    'LINK_REGEX',
    'LINK_TYPE_REGEX',
    'OEMBED_LOGGER',
    'P_REGEX',
    'SANDBOX',
    'URL_PATTERNS',
    'WIDTH'
]



from django.conf.urls import url
from .views import OEmbedProviderView, OEmbedAJAXView


urlpatterns = [
    url(r'^provider/$', OEmbedProviderView.as_view(), name='oembed_provider'),
    url(r'^object\.json$', OEmbedAJAXView.as_view(), name='oembed_object')
]
